@@ -4,45 +4,40 @@
 from __future__ import absolute_import, division, print_function
 
 import asyncio
-from glob import has_magic
 import io
 import logging
 import os
+import re
 import warnings
 import weakref
+from datetime import datetime, timedelta
+from glob import has_magic
 
 from azure.core.exceptions import (
     ClientAuthenticationError,
     HttpResponseError,
-    ResourceNotFoundError,
     ResourceExistsError,
+    ResourceNotFoundError,
 )
-from azure.storage.blob._shared.base_client import create_configuration
 from azure.datalake.store import AzureDLFileSystem, lib
 from azure.datalake.store.core import AzureDLFile, AzureDLPath
-from azure.storage.blob.aio import BlobServiceClient as AIOBlobServiceClient
-from azure.storage.blob import generate_blob_sas, BlobSasPermissions
-from azure.storage.blob.aio._list_blobs_helper import BlobPrefix
+from azure.storage.blob import BlobSasPermissions, generate_blob_sas
 from azure.storage.blob._models import BlobBlock, BlobProperties, BlobType
+from azure.storage.blob._shared.base_client import create_configuration
+from azure.storage.blob.aio import BlobServiceClient as AIOBlobServiceClient
+from azure.storage.blob.aio._list_blobs_helper import BlobPrefix
 from fsspec import AbstractFileSystem
-from fsspec.asyn import (
-    sync,
-    AsyncFileSystem,
-    get_loop,
-    sync_wrapper,
-    get_running_loop,
-)
+from fsspec.asyn import AsyncFileSystem, get_loop, get_running_loop, sync, sync_wrapper
 from fsspec.spec import AbstractBufferedFile
 from fsspec.utils import infer_storage_options, tokenize
+
 from .utils import (
+    close_container_client,
+    close_service_client,
     filter_blobs,
     get_blob_metadata,
-    close_service_client,
-    close_container_client,
 )
 
-from datetime import datetime, timedelta
-
 logger = logging.getLogger(__name__)
 
 FORWARDED_BLOB_PROPERTIES = [
@@ -83,9 +78,9 @@ class AzureDatalakeFileSystem(AbstractFileSystem):
 
     Examples
     --------
+
     >>> adl = AzureDatalakeFileSystem(tenant_id="xxxx", client_id="xxxx",
     ...                               client_secret="xxxx")
-
     >>> adl.ls('')
 
     Sharded Parquet & CSV files can be read as
@@ -241,7 +236,7 @@ class AzureDatalakeFile(AzureDLFile):
         path,
         mode="rb",
         autocommit=True,
-        block_size=2 ** 25,
+        block_size=2**25,
         cache_type="bytes",
         cache_options=None,
         *,
@@ -362,33 +357,36 @@ class AzureBlobFileSystem(AsyncFileSystem):
 
     Examples
     --------
+
     Authentication with an account_key
-    >>> abfs = AzureBlobFileSystem(account_name="XXXX", account_key="XXXX", container_name="XXXX")
+
+    >>> abfs = AzureBlobFileSystem(account_name="XXXX", account_key="XXXX")
     >>> abfs.ls('')
 
-    **  Sharded Parquet & csv files can be read as: **
-        ------------------------------------------
-        ddf = dd.read_csv('abfs://container_name/folder/*.csv', storage_options={
-        ...    'account_name': ACCOUNT_NAME, 'account_key': ACCOUNT_KEY})
-
-        ddf = dd.read_parquet('abfs://container_name/folder.parquet', storage_options={
-        ...    'account_name': ACCOUNT_NAME, 'account_key': ACCOUNT_KEY,})
-
     Authentication with an Azure ServicePrincipal
+
     >>> abfs = AzureBlobFileSystem(account_name="XXXX", tenant_id=TENANT_ID,
-        ...    client_id=CLIENT_ID, client_secret=CLIENT_SECRET)
+    ...                            client_id=CLIENT_ID, client_secret=CLIENT_SECRET)
     >>> abfs.ls('')
 
     Authentication with DefaultAzureCredential
+
     >>> abfs = AzureBlobFileSystem(account_name="XXXX", anon=False)
     >>> abfs.ls('')
 
-    **  Read files as: **
-        -------------
-        ddf = dd.read_csv('abfs://container_name/folder/*.csv', storage_options={
-            'account_name': ACCOUNT_NAME, 'tenant_id': TENANT_ID, 'client_id': CLIENT_ID,
-            'client_secret': CLIENT_SECRET})
-        })
+    Read files as
+
+    >>> ddf = dd.read_csv('abfs://container_name/folder/*.csv', storage_options={
+    ...     'account_name': ACCOUNT_NAME, 'tenant_id': TENANT_ID, 'client_id': CLIENT_ID,
+    ...     'client_secret': CLIENT_SECRET})
+    ... })
+
+    Sharded Parquet & csv files can be read as:
+
+    >>> ddf = dd.read_csv('abfs://container_name/folder/*.csv', storage_options={
+    ...                   'account_name': ACCOUNT_NAME, 'account_key': ACCOUNT_KEY})
+    >>> ddf = dd.read_parquet('abfs://container_name/folder.parquet', storage_options={
+    ...                       'account_name': ACCOUNT_NAME, 'account_key': ACCOUNT_KEY,})
     """
 
     protocol = "abfs"
@@ -491,6 +489,21 @@ class AzureBlobFileSystem(AsyncFileSystem):
         logger.debug(f"_strip_protocol({path}) = {ops}")
         return ops["path"]
 
+    @staticmethod
+    def _get_kwargs_from_urls(urlpath):
+        """Get the account_name from the urlpath and pass to storage_options"""
+        ops = infer_storage_options(urlpath)
+        out = {}
+        host = ops.get("host", None)
+        if host:
+            match = re.match(
+                r"(?P<account_name>.+)\.(dfs|blob)\.core\.windows\.net", host
+            )
+            if match:
+                account_name = match.groupdict()["account_name"]
+                out["account_name"] = account_name
+        return out
+
     def _get_credential_from_service_principal(self):
         """
         Create a Credential for authentication.  This can include a TokenCredential
@@ -1714,7 +1727,7 @@ class AzureBlobFileSystem(AsyncFileSystem):
 class AzureBlobFile(AbstractBufferedFile):
     """File-like operations on Azure Blobs"""
 
-    DEFAULT_BLOCK_SIZE = 5 * 2 ** 20
+    DEFAULT_BLOCK_SIZE = 5 * 2**20
 
     def __init__(
         self,
@@ -1930,6 +1943,14 @@ class AzureBlobFile(AbstractBufferedFile):
 
     _initiate_upload = sync_wrapper(_async_initiate_upload)
 
+    def _get_chunks(self, data, chunk_size=1024**3):  # Keeping the chunk size as 1 GB
+        start = 0
+        length = len(data)
+        while start < length:
+            end = min(start + chunk_size, length)
+            yield data[start:end]
+            start = end
+
     async def _async_upload_chunk(self, final: bool = False, **kwargs):
         """
         Write one part of a multi-block file upload
@@ -1947,11 +1968,18 @@ class AzureBlobFile(AbstractBufferedFile):
         block_id = f"{block_id:07d}"
         if self.mode == "wb":
             try:
-                async with self.container_client.get_blob_client(blob=self.blob) as bc:
-                    await bc.stage_block(
-                        block_id=block_id, data=data, length=length,
-                    )
-                self._block_list.append(block_id)
+                for chunk in self._get_chunks(data):
+                    async with self.container_client.get_blob_client(
+                        blob=self.blob
+                    ) as bc:
+                        await bc.stage_block(
+                            block_id=block_id,
+                            data=chunk,
+                            length=len(chunk),
+                        )
+                        self._block_list.append(block_id)
+                        block_id = len(self._block_list)
+                        block_id = f"{block_id:07d}"
 
                 if final:
                     block_list = [BlobBlock(_id) for _id in self._block_list]
